@@ -1,6 +1,6 @@
 /*
  * curl.c
- * $Id: curl.c 30972 2007-11-12 17:11:33Z jmpp@macports.org $
+ * $Id: curl.c 52758 2009-06-22 22:27:02Z jmr@macports.org $
  *
  * Copyright (c) 2005 Paul Guyot, The MacPorts Project.
  * All rights reserved.
@@ -34,14 +34,11 @@
 #include <config.h>
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
-
-#if HAVE_STRING_H
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
-#endif
 
 #ifdef HAVE_UTIME_H
 #include <utime.h>
@@ -50,25 +47,16 @@
 #include <curl/curl.h>
 
 #include <tcl.h>
-#include <tclDecls.h>
 
 #include "curl.h"
 
-/* Avoid a warning with Tcl < 8.4, even if Tcl_GetIndexFromObj's tablePtr
-probably isn't modified. */
-#if (TCL_MAJOR_VERSION > 8) || (TCL_MINOR_VERSION >= 4)
-typedef CONST char* tableEntryString;
-#else
-typedef char* tableEntryString;
-#endif
-
 /*
  * Some compiled-in constants that we may wish to change later, given more
  * empirical data.  These represent "best guess" values for now.
  */
-#define _CURL_CONNECTION_TIMEOUT	((long)(5 * 60))	/* 5 minutes */
+#define _CURL_CONNECTION_TIMEOUT	((long)(30))		/* 30 seconds */
 #define _CURL_MINIMUM_XFER_SPEED	((long)1024)		/* 1Kb/sec */
-#define _CURL_MINIMUM_XFER_TIMEOUT	((long)(10 * 60))	/* 10 minutes */
+#define _CURL_MINIMUM_XFER_TIMEOUT	((long)(60))		/* 1 minute */
 
 /* ========================================================================= **
  * Definitions
@@ -81,6 +69,7 @@ typedef char* tableEntryString;
 int SetResultFromCurlErrorCode(Tcl_Interp* interp, CURLcode inErrorCode);
 int CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
 int CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
+int CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
 
 /* ========================================================================= **
  * Entry points
@@ -108,18 +97,16 @@ SetResultFromCurlErrorCode(Tcl_Interp* interp, CURLcode inErrorCode)
 			theResult = TCL_OK;
 			break;
 		
-		default:
-#ifdef HAVE_CURL_EASY_STRERROR
-			Tcl_SetResult(interp, (char*) curl_easy_strerror(inErrorCode), TCL_VOLATILE);
+		default: {
+#if LIBCURL_VERSION_NUM >= 0x070c00 /* 7.12.0 */
+			Tcl_SetResult(interp, (char *)curl_easy_strerror(inErrorCode), TCL_VOLATILE);
 #else
-			{
-				char theErrorString[512];
-				(void) snprintf(theErrorString, sizeof(theErrorString),
-					"curl error %i", inErrorCode);
-				Tcl_SetResult(interp, theErrorString, TCL_VOLATILE);				
-			}
+			char theErrorString[512];
+			(void)snprintf(theErrorString, sizeof(theErrorString), "curl error %i", inErrorCode);
+			Tcl_SetResult(interp, theErrorString, TCL_VOLATILE);
 #endif
 			theResult = TCL_ERROR;
+		}
 	}
 	
 	return theResult;
@@ -150,7 +137,7 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		const char* theUserPassString = NULL;
 		const char* effectiveURLVarName = NULL;
 		char* effectiveURL = NULL;
-		char* userAgent = "MacPorts libcurl/" LIBCURL_VERSION;
+		char* userAgent = PACKAGE_STRING " libcurl/" LIBCURL_VERSION;
 		int optioncrsr;
 		int lastoption;
 		const char* theURL;
@@ -249,6 +236,7 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		if (theFile == NULL) {
 			Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
 			theResult = TCL_ERROR;
+                        break;
 		}
 
 		/* Create the CURL handle */
@@ -268,6 +256,13 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			break;
 		}
 
+		/* --max-redirs option, same default as curl command line */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_MAXREDIRS, 50);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
 		/* -f option */
 		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_FAILONERROR, 1);
 		if (theCurlCode != CURLE_OK) {
@@ -359,6 +354,45 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		(void) fclose( theFile );
 		theFile = NULL;
 		
+#if LIBCURL_VERSION_NUM == 0x070d01 /* work around broken Tiger version of cURL */
+		if (remotetime) {
+			FILE *fp;
+			char *tmp, *p;
+			char buf[BUFSIZ];
+			size_t size;
+			
+			tmp = tmpnam(NULL);
+			fp = fopen(tmp, "w");
+			if (fp == NULL) {
+				Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
+				theResult = TCL_ERROR;
+				break;
+			}
+			theFile = fopen( theFilePath, "r");
+			if (theFile == NULL) {
+				Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
+				theResult = TCL_ERROR;
+				break;
+			}
+			if ( (p = fgets(buf, BUFSIZ, theFile)) != NULL) {
+				/* skip stray header escaping into output */
+				if (strncmp(p, "Last-Modified:", 14) != 0)
+					rewind(theFile);
+			}
+			while ( (size = fread(buf, 1, BUFSIZ, theFile)) > 0) {
+				fwrite(buf, 1, size, fp);
+			}
+			(void) fclose( theFile );
+			theFile = NULL;
+			fclose(fp);
+			if (rename(tmp, theFilePath) != 0) {
+				Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
+				theResult = TCL_ERROR;
+				break;
+			}
+		}
+#endif
+
 #ifdef HAVE_UTIME_H
 		if (remotetime) {
 			theCurlCode = curl_easy_getinfo(theHandle, CURLINFO_FILETIME, &theFileTime);
@@ -459,6 +493,7 @@ CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		if (theFile == NULL) {
 			Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
 			theResult = TCL_ERROR;
+                        break;
 		}
 
 		/* Create the CURL handle */
@@ -478,6 +513,13 @@ CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			break;
 		}
 
+		/* --max-redirs option, same default as curl command line */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_MAXREDIRS, 50);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
 		/* -f option */
 		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_FAILONERROR, 1);
 		if (theCurlCode != CURLE_OK) {
@@ -631,6 +673,7 @@ CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		if (theFile == NULL) {
 			Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
 			theResult = TCL_ERROR;
+                        break;
 		}
 
 		/* Create the CURL handle */
@@ -650,6 +693,13 @@ CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			break;
 		}
 
+		/* --max-redirs option, same default as curl command line */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_MAXREDIRS, 50);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
 		/* -f option */
 		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_FAILONERROR, 1);
 		if (theCurlCode != CURLE_OK) {
@@ -766,7 +816,7 @@ CurlCmd(
     	kCurlGetSize
     } EOption;
     
-	static tableEntryString options[] = {
+	static const char *options[] = {
 		"fetch", "isnewer", "getsize", NULL
 	};
 	int theResult = TCL_OK;
@@ -815,8 +865,3 @@ CurlInit(Tcl_Interp* interp)
 	CURLcode theCurlCode = curl_global_init(CURL_GLOBAL_ALL);
 	return SetResultFromCurlErrorCode(interp, theCurlCode);
 }
-
-/* ============================================================== **
-** As of next Thursday, UNIX will be flushed in favor of TOPS-10. **
-** Please update your programs.                                   **
-** ============================================================== */
